/* ----------------------------------------
 * Origin  : GFI INFORMATIQUE 
 * Project : CDPP Propagation Tool 
 * ----------------------------------------
 * History 
 * Version : 01/05/2015 : SCA : V2.1
 * ----------------------------------------
 */
package fr.tlse.irap.pt.client.controller.eclipticPlane;

import fr.tlse.irap.pt.client.controller.common.ISpaceElementMenu;

public interface IEclipticPlane extends ISpaceElementMenu {

	public void runShowCME();
	public void runShowJMapFOV();
	public void runShowFOV(String probeName, String camera);
	public void runShowAllFOVs(String name);
	public void runReset();
	public void searchComets();
	public void runShowSpiral(String spiralType);
//	public void runShowMap(String type);
	public void runShowShocks();
}
